﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

// Importe für WCF
using System.ServiceModel;
using System.Runtime.Serialization;

// Importe für Kontrakte und Implementierung
using Rainbird.Examples.WcfDemo.Contracts;
using Rainbird.Examples.WcfDemo.Implementation;
using Rainbird.Examples.WcfDemo.Implementation.AddressDataSetTableAdapters;

namespace Rainbird.Examples.WcfDemo.Services
{
    /// <summary>
    /// Adressdienst.
    /// </summary>
    public class AddressService : IAddressService
    {
        #region IAddressService Member

        /// <summary>
        /// Gibt alle Adressen zurück.
        /// </summary>
        /// <returns>Tabelle mit Adressen</returns>
        public AddressDataSet GetAllAddresses()
        {
            // Tabellenadapter für Zugriff auf Adressen erzeugen
            using (AddressTableAdapter adapter = new AddressTableAdapter())
            {
                // DataSet erzeugen
                AddressDataSet dataSet = new AddressDataSet();

                // Alle Adressen abrufenen
                adapter.Fill(dataSet.Address);

                // DataSet zurückgeben
                return dataSet;
            }
        }

        /// <summary>
        /// Gibt alle Adressen zurück, deren Nachname einem bestimmten Suchmuster entspricht.
        /// </summary>
        /// <param name="searchPattern">Suchmuster</param>
        /// <returns>Tabelle mit Adressen</returns>
        public AddressDataSet FindAddressesByLastName(string searchPattern)
        {           
            // Tabellenadapter für Zugriff auf Adressen erzeugen
            using (AddressTableAdapter adapter = new AddressTableAdapter())
            {
                // DataSet erzeugen
                AddressDataSet dataSet = new AddressDataSet();

                // Adressen laut Suchmuster abrufen
                adapter.FillByLastNameLike(dataSet.Address,searchPattern);

                // DataSet zurückgeben
                return dataSet;                
            }
        }

        /// <summary>
        /// Speichert Adressen.
        /// </summary>
        /// <param name="changes">Tabelle mit zu speichernden Adressen</param>
        /// <returns>Gespeicherte Tabelle mit Adressen</returns>
        public AddressDataSet SaveAddresses(AddressDataSet changes)
        {
            // Tabellenadapter für Zugriff auf Adressen erzeugen
            using (AddressTableAdapter adapter = new AddressTableAdapter())
            {  
                // Adressen speichern
                adapter.Update(changes);
                
                // Gespeicherte Tabelle zurückgeben
                return changes;
            }
        }

        #endregion
    }
}
