﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Rainbird.Examples.WcfDemo.Contracts;

namespace Rainbird.Examples.WcfDemo.WinClient
{
    /// <summary>
    /// Formular zur Verwaltung von Adressen.
    /// </summary>
    public partial class MainForm : Form
    {
        #region Deklarationen

        // Stellvertreter-Objekt für den Zugriff auf den Adressdienst
        AddressServiceProxy.IAddressService _addressServiceProxy = null;

        #endregion

        #region Konstruktor

        /// <summary>
        /// Standardkonstruktor.
        /// </summary>
        public MainForm()
        {
            // Steuerelemente vorbereiten
            InitializeComponent();
        }

        #endregion

        #region Ereignisprozeduren

        /// <summary>
        /// Ereignisprozedur: Beim Laden des Formulars.
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void MainForm_Load(object sender, EventArgs e)
        {
            // Verbindung zum Adressdienst herstellen
            _addressServiceProxy = new AddressServiceProxy.AddressServiceClient();
        }

        /// <summary>
        /// Ereignisprozedur: Beim Schließen des Formulars.
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void MainForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            // Wenn die Verbindung zum Adressdienst noch besteht ...
            if (_addressServiceProxy != null)
                // Verbindung schließen
                ((AddressServiceProxy.AddressServiceClient)_addressServiceProxy).Close();

        }

        /// <summary>
        /// Ereignisprozedur: Bei Klick auf "Alle Adressen vom Server abrufen".
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void _getAllAddressesTool_Click(object sender, EventArgs e)
        {
            // Alle Adressen laden
            LoadAllAddresses();
        }

        /// <summary>
        /// Ereignisprozedur: Bei Klick auf "Neu".
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void _newAddressTool_Click(object sender, EventArgs e)
        {
            // Adresse erzeugen
            CreateAddress();
        }

        /// <summary>
        /// Ereignisprozedur: Bei Klick auf "Speichern".
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void _saveTool_Click(object sender, EventArgs e)
        {
            // Speichern
            SaveAddresses();
        }

        /// <summary>
        /// Ereignisprozedur: Bei Klick auf "Suchen".
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void _searchTool_Click(object sender, EventArgs e)
        {
            // Suchen
            FindAddressesByLastName();
        }

        #endregion

        #region Anzeigelogik

        /// <summary>
        /// Lädt alle Adressen vom Server.
        /// </summary>
        private void LoadAllAddresses()
        {
            // Sanduhr anzeigen
            this.Cursor = Cursors.WaitCursor;

            // Lokales DataSet leeren
            _addressDataSet.Clear();

            // Adressen vom Server aufrufen und ins lokale DataSet laden
            _addressDataSet.Merge(_addressServiceProxy.GetAllAddresses());

            // Mauspfeil anzeigen
            this.Cursor = Cursors.Default;
        }

        /// <summary>
        /// Erstellt eine neue Adresse.
        /// </summary>
        private void CreateAddress()
        {
            // Neue Adresse erzeugen
            AddressDataSet.AddressRow address = _addressDataSet.Address.NewAddressRow();

            // Schlüssel vergeben
            address.AddressID = Guid.NewGuid();

            // Datensatz anfügen
            _addressDataSet.Address.AddAddressRow(address);

            // Zum letzen Datensatz navigieren
            _addressBinding.MoveNext();
        }

        /// <summary>
        /// Speichert Änderungen an Adressen.
        /// </summary>
        private void SaveAddresses()
        {
            // Sanduhr anzeigen
            this.Cursor = Cursors.WaitCursor;

            // Datenbindung anweisen, ausstehende Änderungen zu übernehmen
            _addressBinding.EndEdit();

            // Adressen persistieren und neu laden
            _addressDataSet.Merge(_addressServiceProxy.SaveAddresses(_addressDataSet));

            // Mauspfeil anzeigen
            this.Cursor = Cursors.Default;
        }

        /// <summary>
        /// Sucht Adressen über den Nachnamen.
        /// </summary>
        private void FindAddressesByLastName()
        {
            // Wenn ein Suchmuster eingegben ist ...
            if (!String.IsNullOrEmpty(_findByLastNameBox.Text))
            {
                // Sanduhr anzeigen
                this.Cursor = Cursors.WaitCursor;

                // Suchmuster abrufen
                string pattern = _findByLastNameBox.Text;

                // Lokales DataSet leeren
                _addressDataSet.Clear();

                // Adressen suchen und anzeigen
                _addressDataSet.Merge(_addressServiceProxy.FindAddressesByLastName(pattern));

                // Mauspfeil anzeigen
                this.Cursor = Cursors.Default;
            }
        }

        #endregion
    }
}
